var dockUpdateTimer;

// items for the rotator.  This will be an array
var m_rotatorItems;

// status for the rotator.  0 = ready, 1 = loading, 2 = failed
var m_rotatorStatus = 0;
var m_inFailure = false;

// current index into the rotator list of items
var m_rotatorIndex = -1;

// opacity value for fading
var m_opacity;

// rotator timer
var m_rotatorTimer;
var m_rotatorInterval = 5000;

// refresh timer
var m_refreshTimer = null;
var m_refreshInterval = 1800000;

// refresh more often if we're in failure
var m_refreshFailureInterval = 150000;

var m_tag = null;

function loadMain()
{
    // localize
    Localize();

    System.Gadget.Flyout.file = "flyout.html";
    System.Gadget.Flyout.onShow = flyoutShowing;
    System.Gadget.Flyout.onHide = flyoutHiding;
    System.Gadget.settingsUI = "settings.html";
    System.Gadget.onSettingsClosed = settingsClosed;
    System.Gadget.onShowSettings = settingsShowing;
    
    System.Gadget.visibilityChanged = VisibilityChanged;

    DoDockUpdate();
    StartRefresh();
}

function settingsClosed(event)
{
    if (event.closeAction == event.Action.commit)
    {
        m_tag = System.Gadget.Settings.readString("tag");
        
        DoDockUpdate();
        StartRefresh();
    }
    else if (event.closeAction == event.Action.cancel && m_tag != "")
    {
        RunRotator();
        StartRefresh();
    }
}

function settingsShowing()
{
    StopRefresh();
    StopRotator();
}

function VisibilityChanged()
{
    // We need to do this in order to re-start timers on resume from sleep
    if (System.Gadget.docked == true)
    {
        if (System.Gadget.visible == true)
        {
            DoDockUpdate();
            StartRefresh();
        }
        else
        {
            StopRefresh();
            StopRotator();
        }
    }
}

function flyoutShowing()
{
    StopRefresh();
    StopRotator();
    docked.style.display = "none";
    dockedFly.style.display = "block";
}

function flyoutHiding()
{
    StartRefresh();
    RunRotator();
    dockedFly.style.display = "none";
    docked.style.display = "block";
}

function Localize()
{
    dockNoResults.innerText = DOCK_NO_RESULTS;
    dockNoTag.innerText = DOCK_NO_TAG;
}

function StartRefresh()
{
    var interval = m_refreshInterval;
    
    if (m_rotatorStatus == 2)
    {
        interval = m_refreshFailureInterval;
    }
    
    if (m_rotatorStatus != 0)
    {
        m_refreshTimer = setInterval("DoDockUpdate()", interval);
    }
}

function StopRefresh()
{
    if (m_refreshTimer != null)
    {
        clearInterval(m_refreshTimer);
        m_refreshTimer = null;
    }
}

function DoDockUpdate()
{
    // only run this if the rotator is not already loading
    if (m_rotatorStatus != 1)
    {
        // do some housekeeping
        if (m_tag == null)
        {
            SetNoTag();
            return;
        }
        SetRotatorLoading();
        // TODO:  respect settings for bungie favorites vs. gamertag
        var url = "http://www.bungie.net/stats/halo3/PlayerScreenshotsRss.ashx?gamertag=" + encodeURI(m_tag);

        // let's load some screenshots!
        var req = new XMLHttpRequest();
        req.open("GET", url);

        req.onreadystatechange = function()
        {
            if (req.readyState == 4)
            {
                // this is evil, but for some reason Bungie's feed won't load automatically
                var doc = new ActiveXObject("MSXML2.DOMDocument.3.0");
                doc.loadXML(req.responseText);
                m_rotatorItems = ParseShots(doc);
                if (m_rotatorItems != null)
                {
                    m_rotatorStatus = 0;
                    
                    // check if we need to reset the refresh status
                    if (m_inFailure == true)
                    {
                        m_inFailure = false;
                        StopRefresh();
                        StartRefresh();
                    }
                    
                    // success!  Show the image and start the rotator
                    DoRotate();
                    
                    RunRotator();
                }
                else
                {
                    // failure.  clean up
                    SetRotatorFailed();
                }
                
            }
        }

        req.send();
        
    }
}

function SetRotatorLoading()
{
    StopRotator();
    dockFaderImg.style.display = "none";
    dockNoResults.style.display = "none";
    dockNoTag.style.display = "none"
    dockRotatorImg.style.display = "none";
    dockRotatorImg.src = "images/loading.gif";
    dockRotatorImg.style.display = "inline";
    m_rotatorStatus = 1;
    m_rotatorIndex = -1;
    ResetRotatorPaging();
    dockRotatorImg.style.cursor = "default";
}

function SetRotatorFailed()
{
    m_rotatorStatus = 2;
    m_inFailure = true;
    dockRotatorImg.style.display = "none";
    dockFaderImg.style.display = "none";
    dockNoResults.style.display = "block";
    dockNoTag.style.display = "none"

    StopRefresh()
    StartRefresh();
}

function SetNoTag()
{
    m_rotatorStatus = 0;
    m_inFailure = false;
    dockRotatorImg.style.display = "none";
    dockFaderImg.style.display = "none";
    dockNoResults.style.display = "none";
    dockNoTag.style.display = "block"
    
    StopRefresh()
    StartRefresh();
}

function ResetRotatorPaging()
{
    m_rotatorIndex = -1;
    m_rotatorItems = null;

    dockBack.src = "images/back_disabled.png";
    dockFwd.src = "images/next_disabled.png";
    dockBack.style.cursor = "default";
    dockFwd.style.cursor = "default";    
}

function SetRotatorPagingBounds()
{
    if (m_rotatorIndex > 0)
    {
        dockBack.src = "images/back.png";
        dockBack.style.cursor = "pointer";
    }
    else
    {
        dockBack.src = "images/back_disabled.png";
        dockBack.style.cursor = "default";
    }
    
    if (m_rotatorIndex < m_rotatorItems.length - 1)
    {
        dockFwd.src = "images/next.png";
        dockFwd.style.cursor = "pointer";
    }
    else
    {
        dockFwd.src = "images/next_disabled.png";
        dockFwd.style.cursor = "default";
    }
}

function ShowImage()
{
    // set pager-ness
    SetRotatorPagingBounds();
    
    // Change the image
    
    // display the image to fade and make sure it's visible
    m_opacity = 100;
    dockFaderImg.style.display = "inline";
    dockFaderImg.style.filter = "alpha(opacity=" + m_opacity + ")";

    // update the real image underneath
    dockRotatorImg.style.cursor = "pointer";
    dockRotatorImg.src = m_rotatorItems[m_rotatorIndex].thumbnail;

    // run the fader
    DoFade();
    
}

function DoRotate()
{
    
    if (m_rotatorStatus == 0)
    {
        // increment the index
        m_rotatorIndex = (m_rotatorIndex + 1) % m_rotatorItems.length;
        
        ShowImage();
    }
}

function RunRotator()
{
    // run the timer every 30 seconds (5 seconds for testing only)
    m_rotatorTimer = setInterval("DoRotate()", m_rotatorInterval);
}

function StopRotator()
{
    if (m_rotatorTimer != null)
    {
        clearInterval(m_rotatorTimer);
        m_rotatorTimer = null;
    }
}

function DoFade()
{
    m_opacity -= 15;
    if (m_opacity > 0)
    {
        dockFaderImg.style.filter = "alpha(opacity=" + m_opacity + ")";
        setTimeout("DoFade()", 100);
    }
    else
    {
        dockFaderImg.style.display = "none";
        dockFaderImg.src = dockRotatorImg.src;
    }
}

function ParseShots(responseXML)
{
    // sanity
    if (responseXML == null || responseXML.documentElement == null)
    {
        return null;
    }

    var xpath = "//rss/channel/item";
    
    // TODO:  bungie favorites require an extra filter on the xpath

    var shots = responseXML.documentElement.selectNodes(xpath);

    if (shots != null && shots.length > 0)
    {
        var collection = new Array(shots.length);

        for (var i = 0; i < shots.length; i++)
        {
            var title = shots[i].selectSingleNode("title").text;
            var description = shots[i].selectSingleNode("description").text;
            var link = shots[i].selectSingleNode("link").text;
            var thumbnail = shots[i].selectSingleNode("halo3:thumbnail_image").text;
            var image = shots[i].selectSingleNode("halo3:medium_size_image").text;

            collection[i] = {
                title: title
                , description: description
                , link: link
                , thumbnail: thumbnail 
                , image: image
            };

            title = description = link = thumbnail = image = null;
        }
        
        shots = null;
        xpath = null;
        return collection;
    }
    else
    {
        return null;
    }
}

function OnRotatorBack()
{
    if (m_rotatorStatus == 0 && m_rotatorIndex > 0)
    {
        StopRotator();
        m_rotatorIndex--;
        ShowImage();
        RunRotator();
    }
}

function OnRotatorFwd()
{
    if (m_rotatorStatus == 0 && m_rotatorIndex < m_rotatorItems.length - 1)
    {
        StopRotator();
        m_rotatorIndex++;
        ShowImage();
        RunRotator();
    }
}

function OnRotatorClick()
{
    if (m_rotatorStatus == 0 && m_rotatorIndex != -1)
    {
        // Tell the flyout to pickup the current items
        m_useCurrent = true;
        
        // Show the flyout
        System.Gadget.Flyout.show = true;
    }
}
